@@ -34,8 +34,8 @@ class Party(ModelSQL, ModelView):
     name = fields.Char('Name', required=True, select=True,
         states=STATES, depends=DEPENDS)
     code = fields.Char('Code', required=True, select=True,
-        order_field="%(table)s.code_length %(order)s, " \
-            "%(table)s.code %(order)s",
+        order_field=("%(table)s.code_length %(order)s, "
+            "%(table)s.code %(order)s"),
         states={
             'readonly': Eval('code_readonly', True),
             },
@@ -77,14 +77,12 @@ class Party(ModelSQL, ModelView):
         super(Party, cls).__setup__()
         cls._sql_constraints = [
             ('code_uniq', 'UNIQUE(code)',
-             'The code of the party must be unique!')
-        ]
-        cls._constraints += [
-            ('check_vat', 'invalid_vat'),
+             'The code of the party must be unique.')
         ]
         cls._error_messages.update({
-            'invalid_vat': 'Invalid VAT number!',
-        })
+                'invalid_vat': ('Invalid VAT number "%(vat)s" on party '
+                    '"%(party)s".'),
+                })
         cls._order.insert(0, ('name', 'ASC'))
 
     @staticmethod
@@ -100,10 +98,9 @@ class Party(ModelSQL, ModelView):
         if Transaction().user == 0:
             return []
         Address = Pool().get('party.address')
-        fields_names = list(x for x in set(Address._fields.keys()
-                + Address._inherit_fields.keys())
-                if x not in ['id', 'create_uid', 'create_date',
-                    'write_uid', 'write_date'])
+        fields_names = list(x for x in Address._fields.keys()
+            if x not in ('id', 'create_uid', 'create_date',
+                'write_uid', 'write_date'))
         return [Address.default_get(fields_names)]
 
     @staticmethod
@@ -152,17 +149,17 @@ class Party(ModelSQL, ModelView):
         return ''
 
     @classmethod
-    def create(cls, values):
+    def create(cls, vlist):
         Sequence = Pool().get('ir.sequence')
         Configuration = Pool().get('party.configuration')
 
-        values = values.copy()
-        if not values.get('code'):
-            config = Configuration(1)
-            values['code'] = Sequence.get_id(config.party_sequence.id)
-
-        values['code_length'] = len(values['code'])
-        return super(Party, cls).create(values)
+        vlist = [x.copy() for x in vlist]
+        for values in vlist:
+            if not values.get('code'):
+                config = Configuration(1)
+                values['code'] = Sequence.get_id(config.party_sequence.id)
+            values['code_length'] = len(values['code'])
+        return super(Party, cls).create(vlist)
 
     @classmethod
     def write(cls, parties, vals):
@@ -179,17 +176,7 @@ class Party(ModelSQL, ModelView):
             default = {}
         default = default.copy()
         default['code'] = None
-        default['addresses'] = None
-        new_parties = []
-        for party in parties:
-            new_party = super(Party, cls).copy(party, default=default)
-            Address.copy([x.id for x in new_party.addresses],
-                    default={
-                        'party': new_party.id,
-                        })
-            new_parties.append(new_party)
-
-        return new_parties
+        return super(Party, cls).copy(parties, default=default)
 
     @classmethod
     def search_rec_name(cls, name, clause):
@@ -219,6 +206,12 @@ class Party(ModelSQL, ModelView):
                 return address
         return default_address
 
+    @classmethod
+    def validate(cls, parties):
+        super(Party, cls).validate(parties)
+        for party in parties:
+            party.check_vat()
+
     def check_vat(self):
         '''
         Check the VAT number depending of the country.
@@ -231,7 +224,7 @@ class Party(ModelSQL, ModelView):
         if not self.vat_country:
             return True
 
-        if not getattr(vatnumber, 'check_vat_' + \
+        if not getattr(vatnumber, 'check_vat_' +
                 self.vat_country.lower())(vat_number):
 
             #Check if user doesn't have put country code in number
@@ -241,8 +234,10 @@ class Party(ModelSQL, ModelView):
                     'vat_number': vat_number,
                     })
             else:
-                return False
-        return True
+                self.raise_user_error('invalid_vat', {
+                        'vat': vat_number,
+                        'party': self.rec_name,
+                        })
 
 
 class PartyCategory(ModelSQL):
@@ -292,9 +287,9 @@ class CheckVIES(Wizard):
     def __setup__(cls):
         super(CheckVIES, cls).__setup__()
         cls._error_messages.update({
-            'vies_unavailable': 'The VIES service is unavailable, ' \
-                    'try again later.',
-            })
+                'vies_unavailable': ('The VIES service is unavailable, '
+                    'try again later.'),
+                })
 
     def transition_check(self):
         Party = Pool().get('party.party')
